/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Axis2BCListener.java 154 19 avr. 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.axis2.listener.soap;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Serializable;
import java.net.URI;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;

import org.apache.axiom.attachments.Attachments;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.client.Options;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.engine.AxisEngine;
import org.apache.axis2.receivers.AbstractMessageReceiver;
import org.apache.axis2.util.Utils;
import org.objectweb.petals.component.common.su.SimpleServiceUnitManager;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.tools.jbicommon.util.StringHelper;
import org.objectweb.petals.tools.jbicommon.util.XMLUtil;
import org.w3c.dom.Document;

/**
 * The message receiver used by the binding component. The SOAP message is
 * forwarded to the JBI endpoint.
 *
 * @version $Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since Petals 1.0
 * @author alouis,wjoseph,chamerling - EBM Websourcing
 *
 */
public class PetalsReceiver extends AbstractMessageReceiver {

    protected static final String ERROR_SERVICE_NOT_FOUND = "Requested service and/or operation not found/available";

    protected static final String ERROR_GET_WSDL = "Error while retrieving the service WSDL";

    protected static final String ERROR_WRONG_MESSAGE_STATUS = "JBI message has wrong status";

    protected static final String ERROR_SERVICE_INVOKATION = "An error occured in service invokation";

    protected static final String ERROR_CREATE_RESPONSE_FROM_DOM = "Error creating a string from a DOM service description";

    protected static final String ERROR_CREATE_STRING_FROM_NM = "Error creating a string from normalized message";

    protected Logger logger;

    protected ComponentContext context;

    protected DeliveryChannel channel;

    protected MessageExchangeFactory exchangeFactory;

    protected SimpleServiceUnitManager bindingSUM;

    /**
     * Creates a new instance of PetalsReceiver
     *
     * @param context
     * @param channel
     * @param bindingSUM
     * @param log
     */
    public PetalsReceiver(ComponentContext context, DeliveryChannel channel,
        SimpleServiceUnitManager bindingSUM, Logger log) {
        super();
        this.channel = channel;
        this.context = context;
        this.logger = log;
        this.bindingSUM = bindingSUM;
        this.exchangeFactory = channel.createExchangeFactory();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.axis2.engine.MessageReceiver#receive(org.apache.axis2.context.MessageContext)
     */
    public final void receive(final MessageContext msgContext) throws AxisFault {
        logger.log(Level.FINE, "Receive SOAP message");

        MessageContext outMsgContext = Utils
            .createOutMessageContext(msgContext);
        outMsgContext.getOperationContext().addMessageContext(outMsgContext);

        ServiceEndpoint jbiEndpoint = findJBIEndpoint(msgContext);
        process(jbiEndpoint, msgContext, outMsgContext);

        // send the response if required
        if (outMsgContext.getEnvelope() != null) {
            AxisEngine engine = new AxisEngine(msgContext.getOperationContext()
                .getServiceContext().getConfigurationContext());

            engine.send(outMsgContext);
        }
    }

    /**
     * Return the JBI ServiceEndpoint according to : <br>{
     * axisService.getTargetNamespace() } axisService.getName() <br>
     * axisService.getEndpoint()
     *
     * @param msgContext
     * @return the corresponding endpoint, null if no one corresponding
     */
    protected ServiceEndpoint findJBIEndpoint(MessageContext msgContext) {
        AxisService axisService = msgContext.getAxisService();
        String endpointName = axisService.getEndpoint();
        QName serviceName = new QName(axisService.getTargetNamespace(),
            endpointName);

        return context.getEndpoint(serviceName, endpointName);
    }

    /**
     * Create a String result from a DOM document
     *
     * @param document
     *            the DOM Document
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public String createStringFromDOMDocument(Document document)
        throws TransformerException {
        return XMLUtil.createStringFromDOMNode(document);
    }

    /**
     * Sends the SOAPEnvelope of the resquest message directly to a JBI service
     * and operation on the JBI container. The response message from the JBI
     * service invocation is transmitted as an outgoing message.
     *
     * @param inMessage
     *            MessageContext for incoming message
     * @param outMessage
     *            MessageContect fot outgoing message
     * @throws AxisFault
     *             error while processing request, send the JBI Exchange,
     *             processing the JBI response, or if the JBI response is a
     *             Fault
     */
    public void process(ServiceEndpoint endpoint, MessageContext inContext,
        MessageContext outContext) throws AxisFault {
        logger.log(Level.FINE, "Process SOAP request");

        // Retrieve message operation and service
        QName operationName = findSOAPOperation(inContext);
        if (operationName == null) {
            throw new AxisFault("Operation not found in message context");
        }

        MessageExchange exchange = null;
        try {
            exchange = createMessageExchange(inContext, endpoint, operationName);

        } catch (AxisFault e) {
            String errorMsg = "Error while transforming SOAP request to JBI MessageExchange";
            logger.log(Level.SEVERE, errorMsg, e);
            throw new AxisFault(errorMsg, e);
        }

        // send the message exchange through JBI
        // As the SOAP/HTTP request wait for a response,
        // use the sendSynchrone MEP
        try {
            channel.sendSync(exchange);
        } catch (MessagingException e) {
            String errorMsg = "Error while sending message through JBI NMR.";
            logger.log(Level.SEVERE, errorMsg, e);
            throw new AxisFault(errorMsg, e);
        }

        // process the response received from JBI NMR. The outcontext is updated
        processJBIResponse(exchange, outContext, getSOAPFactory(inContext));
    }

    /**
     * Return the operation according to the WSAAction SOAP attribute
     *
     * @param inContext
     *            SOAPrequest context
     * @return operation as QName, null if no WSAAction is specified
     */
    protected QName findSOAPOperation(MessageContext inContext) {
        String operation = inContext.getWSAAction();
        QName operationName = null;
        if (!StringHelper.isNullOrEmpty(operation)) {
            operationName = QName.valueOf(operation);
        }
        return operationName;
    }

    /**
     * Process the JBI response. <br>
     * According to the response status :
     * <li>
     * <ul>
     * do nothin for DONE,
     * </ul>
     * <ul>
     * throws an AxisFault for ERROR,
     * </ul>
     * <ul>
     * set outContext body to a FAULT,
     * </ul>
     * <ul>
     * set outContext body to the response.
     * </ul>
     * </li>
     *
     * @param exchange
     *            JBI response
     * @param outMessage
     *            set to FAULT or OUT JBI response
     * @param factory
     *            soap factory used to create body response
     * @throws AxisFault
     *             with ERROR JBI status, or problem while creating response
     */
    protected void processJBIResponse(MessageExchange exchange,
        MessageContext outMessage, SOAPFactory factory) throws AxisFault {
        if (exchange.getStatus().equals(ExchangeStatus.DONE)) {
            // exchange DONE, nothing to do, no soap response

        } else if (exchange.getStatus().equals(ExchangeStatus.ERROR)) {
            // exchange ERROR: throws an axis fault
            throw new AxisFault(exchange.getError());

        } else if (exchange.getStatus().equals(ExchangeStatus.ACTIVE)) {
            if (exchange.getFault() != null) {
                // exchange FAULT : return a fault message
                try {
                    outMessage.setEnvelope(createSOAPResponse(
                        factory, exchange.getFault()));
                } catch (AxisFault e) {
                    String errorMsg = "Error while creating SOAP Fault response.";
                    logger.log(Level.SEVERE, errorMsg, e);
                    throw new AxisFault(errorMsg, e);
                }

            } else {
                // Get the ouput message if the message is an instance of out
                try {
                    NormalizedMessage nm = null;
                    if (exchange instanceof InOptionalOut) {
                        InOptionalOut mex = (InOptionalOut) exchange;
                        nm = mex.getOutMessage();
                    } else if (exchange instanceof InOut) {
                        InOut mex = (InOut) exchange;
                        nm = mex.getOutMessage();
                    }

                    outMessage.setEnvelope(createSOAPResponse(
                        factory, nm));

                    copyJBIAttachmentsToSOAP(nm, outMessage);
                } catch (AxisFault e) {
                    String errorMsg = "Error while creating SOAP Fault response.";
                    logger.log(Level.SEVERE, errorMsg, e);
                    throw new AxisFault(errorMsg, e);
                }
            }
            // close the messageExchange with the JBI NMR
            try {
                exchange.setStatus(ExchangeStatus.DONE);
                channel.send(exchange);
            } catch (MessagingException e) {
                String errorMsg = "Error while closing JBI MessageExchange.";
                logger.log(Level.SEVERE, errorMsg, e);
                throw new AxisFault(errorMsg, e);
            }

        } else {
            throw new AxisFault(ERROR_WRONG_MESSAGE_STATUS);
        }

    }

    /**
     * Creates a SOAP response from a NormalizedMessage
     *
     * @param factory
     *            soap factory
     * @param nm2
     *            NormalizedMessage containing the response
     * @return a SOAPEnveloppe created from the nm NomalizedMessage content
     * @throws AxisFault
     */
    protected SOAPEnvelope createSOAPResponse(
        SOAPFactory factory, NormalizedMessage nm) throws AxisFault {

        SOAPEnvelope responseEnv = createSOAPEnvelope(factory);

        /*
         * Create and fill the Soap body with the content of the Normalized
         * message
         */
        String response = null;
        if (nm != null) {
            try {
                response = createStringContent(nm);
            } catch (Exception e) {
                logger.log(Level.SEVERE, ERROR_CREATE_STRING_FROM_NM);
                throw new AxisFault(ERROR_CREATE_STRING_FROM_NM, e);
            }
        } else {
            response = "<Response>Done</Response>";
        }
        createSOAPBody(factory, responseEnv, response);
        return responseEnv;
    }

    /**
     * Create a new Axis 2 SOAP body with the right name spaces and service name
     *
     * @param factory
     *            the soap factory
     * @param envelope
     *            a soap envelope
     * @param response
     *            the response
     * @return a new soap body
     */
    protected OMElement createSOAPBody(SOAPFactory factory,
        SOAPEnvelope envelope, String response) throws AxisFault {
        OMElement bodyContent = null;
        ByteArrayInputStream inputStream = new ByteArrayInputStream(response
            .getBytes());
        try {
            XMLStreamReader parser = XMLInputFactory.newInstance()
                .createXMLStreamReader(inputStream);
            StAXOMBuilder builder = new StAXOMBuilder(factory, parser);
            bodyContent = (OMElement) builder.getDocumentElement();
        } catch (Exception e) {
            throw new AxisFault(
                "Error parsing the response from JBI service to a SOAPBody", e);
        }
        SOAPBody respBody = factory.createSOAPBody(envelope);
        respBody.addChild(bodyContent);
        return respBody;
    }

    /**
     * Create a new Axis 2 SOAP envelope with the right name spaces
     *
     * @param factory
     *            the soap factory; there are ex
     * @param namespaces
     *            name spaces for the soap envelope
     * @return a new SOAP envelope
     */
    protected SOAPEnvelope createSOAPEnvelope(SOAPFactory factory) {
        return factory.createSOAPEnvelope();
    }

    /**
     * Create the message exchange
     *
     * @param inContext
     * @param endpoint
     * @param operationName
     * @return
     * @throws AxisFault
     */
    protected MessageExchange createMessageExchange(MessageContext inContext,
        ServiceEndpoint endpoint, QName operationName) throws AxisFault {
        InOptionalOut exchange = null;
        NormalizedMessage nm = null;

        try {

            // create an inoptional out message
            // TODO : get good MEP (feature request 306593)
            Source source = createSourceContent(inContext);
            exchange = exchangeFactory.createInOptionalOutExchange();
            nm = exchange.createMessage();
            exchange.setInMessage(nm);
            nm.setContent(source);
            exchange.setEndpoint(endpoint);
            exchange.setOperation(operationName);

            // add SOAP attachements to normalized message
            copySOAPAttachmentsToJBI(inContext, nm);

            // Get the options from the SOAP message and put them into the JBI
            // message
            mapProperties(inContext, exchange);

        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }

        return exchange;
    }

    /**
     * Create the message to be send through JBI. TODO : get good MEP (feature
     * request 306593). Will be used when mep will be retrieved from WSDL
     * description.
     *
     * @param inContext
     * @param endpoint
     * @param operationName
     * @return
     * @throws AxisFault
     */
    protected MessageExchange createMessageExchange(MessageContext inContext,
        ServiceEndpoint endpoint, QName operationName, URI mep)
        throws AxisFault {
        MessageExchange exchange = null;
        NormalizedMessage nm = null;

        try {
            // create an inoptional out message
            Source source = createSourceContent(inContext);

            exchange = exchangeFactory.createExchange(mep);
            exchange.setEndpoint(endpoint);
            exchange.setOperation(operationName);

            nm = exchange.createMessage();
            nm.setContent(source);

            if (exchange instanceof InOut) {
                InOut mex = (InOut) exchange;
                mex.setInMessage(nm);
            } else if (exchange instanceof InOptionalOut) {
                InOptionalOut mex = (InOptionalOut) exchange;
                mex.setInMessage(nm);
            } else if (exchange instanceof InOut) {
                InOut mex = (InOut) exchange;
                mex.setInMessage(nm);
            } else if (exchange instanceof RobustInOnly) {
                RobustInOnly mex = (RobustInOnly) exchange;
                mex.setInMessage(nm);
            }

            // add SOAP attachements to normalized message
            Attachments attachments = inContext.getAttachmentMap();
            if (attachments != null) {
                String[] ids = attachments.getAllContentIDs();
                for (String id : ids) {
                    logger.log(Level.INFO, "Adding attachment");
                    nm.addAttachment(id, attachments.getDataHandler(id));
                }
            }
            // SOAPHelper.copyAttachmentsFromSOAPToJBI(inContext, nm);

            // Get the options from the SOAP message and put them into the JBI
            // message
            mapProperties(inContext, exchange);

        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }

        return exchange;
    }

    /**
     * Add the options properties from the Axis message context to the
     * properties of the normalized message.
     *
     * @param context
     * @param message
     */
    protected void mapProperties(MessageContext context,
        MessageExchange exchange) {
        Options options = context.getOptions();
        Map properties = options.getProperties();
        for (Object key : properties.keySet()) {
            Object value = properties.get(key);
            if (value instanceof Serializable) {
                exchange.setProperty(key.toString(), properties.get(key));
            }
        }
    }

    /**
     * Create a new Source object containing the axis message Soap envelope
     *
     * @param message
     *            the axis message
     * @return the source content
     */
    private Source createSourceContent(MessageContext message) throws Exception {
        String source = getBodyElement(message);
        return SourceHelper.createSource(source);
    }

    /**
     * Create a new String containing the normalized message content
     *
     * @param message
     *            the normalized message
     * @return the String content
     * @throws Exception
     */
    private String createStringContent(NormalizedMessage message)
        throws Exception {
        return SourceHelper.createString(message.getContent());
    }

    /**
     * Get the body from the SOAP message context.
     *
     * @param message
     * @return
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    private String getBodyElement(MessageContext message) throws Exception {
        String result = new String();
        SOAPEnvelope env = message.getEnvelope();
        SOAPBody body = env.getBody();
        OMNamespace namespace = env.getNamespace();
        Iterator<OMNamespace> envNS = env.getAllDeclaredNamespaces();
        Iterator<OMNamespace> bodyNS = body.getAllDeclaredNamespaces();
        OMElement element = body.getFirstElement();
        if (element != null) {
            element.declareNamespace(namespace);
            while (envNS.hasNext()) {
                element.declareNamespace(envNS.next());
            }
            while (bodyNS.hasNext()) {
                element.declareNamespace(bodyNS.next());
            }
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            element.serialize(os);
            result = os.toString();
        }
        return result;
    }

    /**
     * Add the attachements from the incoming soap request to the Normalized
     * message
     *
     * @param messageContext
     * @param nm
     */
    private void copySOAPAttachmentsToJBI(MessageContext messageContext,
        NormalizedMessage nm) throws MessagingException {

        if (nm != null && messageContext != null) {
            Attachments attachments = messageContext.getAttachmentMap();
            if (attachments != null) {
                Set set = attachments.getContentIDSet();
                for (Object object : set) {
                    String id = (String) object;
                    DataHandler dh = attachments.getDataHandler(id);
                    nm.addAttachment(id, dh);
                }
            }
        }
    }

    /**
     * Add the normalized message attachments to the Axis message context.
     *
     * @param nm
     * @param messageContext
     *
     * @throws AxisFault
     */
    private void copyJBIAttachmentsToSOAP(NormalizedMessage nm,
        MessageContext messageContext) {
        if (nm != null && messageContext != null) {
            Set attachmentNames = nm.getAttachmentNames();
            for (Object id : attachmentNames) {
                String name = (String) id;
                messageContext.addAttachment(name, nm.getAttachment(name));
            }
        }
    }

}
