/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.orchestra.incoming;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;

import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.bc.ExternalListenerManager;
import org.objectweb.petals.engine.orchestra.OrchestraBC;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

/**
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class OrchestraExternalListenerManager implements
    ExternalListenerManager {

    private OrchestraBC orchestraBC;

    private Logger logger;

    private Map<String, JMSConnection> jmsConsumerConnections;

    /**
     * 
     * Creates a new instance of {@link OrchestraExternalListenerManager}
     * 
     * @param logger
     */
    public OrchestraExternalListenerManager(OrchestraBC orchestraBC,
        Map<String, JMSConnection> jmsConsumerConnections) {
        this.orchestraBC = orchestraBC;
        this.logger = orchestraBC.getLogger();
        this.jmsConsumerConnections = jmsConsumerConnections;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.bc.ExternalListenerManager#startListening(java.lang.String,
     *      org.objectweb.petals.tools.jbicommon.descriptor.Extensions)
     */
    public void startListening(String address, Extensions extensions)
        throws PEtALSComponentSDKException {
        logger.log(Level.INFO, "Starting listening on " + address);

        // the connection has already been created, get it from map
        JMSConnection conn = jmsConsumerConnections.get(address);
        OrchestraJMSMessageListener jmsMessageListener = null;
        try {
            jmsMessageListener = new OrchestraJMSMessageListener(orchestraBC);
        } catch (MessagingException e1) {
            throw new PEtALSComponentSDKException(
                "Error during JMS message listener instanciation", e1);
        }
        
        // add JMS listener to connection
        MessageConsumer consumer = conn.getMessageConsumer();
        try {
            consumer.setMessageListener(jmsMessageListener);
        } catch (JMSException e) {
            throw new PEtALSComponentSDKException(
                "Can not set the jms messageListener fro the jms consumer.", e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.bc.ExternalListenerManager#stopListening(java.lang.String)
     */
    public void stopListening(String address)
        throws PEtALSComponentSDKException {
        logger.log(Level.INFO, "Stopping listening on " + address);
        
        // get and stop the connection
        JMSConnection conn = jmsConsumerConnections.get(address);
        conn.stop();
    }

}
