/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.orchestra.util;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Reader;
import java.io.Serializable;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

/**
 * A helper class which provides a JAXP {@link Source} from a String which can
 * be read as many times as required.
 * 
 * @author Christophe HAMERLING - eBWebSourcing
 * 
 */
public class StringSource extends StreamSource implements Serializable {

    /**
	 * 
	 */
	private static final long serialVersionUID = 875836899657755428L;

	private String text;

    private String encoding = "UTF-8";

    /**
     * Creates a new instance of {@link StringSource}
     * 
     * @param text
     */
    public StringSource(String text) {
        this.text = text;
    }

    /**
     * Creates a new instance of {@link StringSource}
     * 
     * @param text
     * @param systemId
     */
    public StringSource(String text, String systemId) {
        this.text = text;
        setSystemId(systemId);
    }

    /**
     * Creates a new instance of {@link StringSource}
     * 
     * @param text
     * @param systemId
     * @param encoding
     */
    public StringSource(String text, String systemId, String encoding) {
        this.text = text;
        this.encoding = encoding;
        setSystemId(systemId);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.xml.transform.stream.StreamSource#getInputStream()
     */
    public InputStream getInputStream() {
        try {
            return new ByteArrayInputStream(text.getBytes(encoding));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.xml.transform.stream.StreamSource#getReader()
     */
    public Reader getReader() {
        return new StringReader(text);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "StringSource[" + text + "]";
    }

    /**
     * 
     * @return
     */
    public String getText() {
        return text;
    }

}
