/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.travelagency.hotel.jms;

import javax.naming.Context;
import javax.naming.InitialContext;

import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.joram.client.jms.admin.User;

import org.objectweb.joram.client.jms.tcp.QueueTcpConnectionFactory;
import org.objectweb.joram.client.jms.tcp.TcpConnectionFactory;

import fr.dyade.aaa.agent.AgentServer;

/**
 * @author Christophe HAMERLING, alouis - eBMWebSourcing
 * 
 */
public class ClassicAdmin {

	public static final String QUEUE_REQUEST = "queueRequest";

	public static final String QUEUE_RESPONSE = "queueResponse";

	public static void main(String[] args) throws Exception {
		System.out.println();
		System.out.println("Start the Hotel JMS services...");
		System.out.println("-------------------------------");
		System.out.println();
		
		startServer();

		initDestinations();

		System.out.println("Start the message receiver...");
		Receiver s = new Receiver();
		s.start();
		System.out.println("Message receiver started...");
		System.out.println();
		System.out.println("... Hotel JMS services started...");
	}

	protected static void initDestinations() throws Exception {
		System.out.println("Initialize JMS destinations...");
		
		System.out.println("   (connect to JORAM server)");
		AdminModule.connect("localhost", 26010, "root", "root", 60);
		System.out.println("   (connected)");
		
		Queue queueRequest = (Queue) Queue.create(QUEUE_REQUEST);
		Queue queueResponse = (Queue) Queue.create(QUEUE_RESPONSE);

		User.create("anonymous", "anonymous");

		queueRequest.setFreeReading();
		queueResponse.setFreeReading();
		queueRequest.setFreeWriting();
		queueResponse.setFreeWriting();
		System.out.println("   (Destinations created)");
		
		javax.jms.ConnectionFactory cf = TcpConnectionFactory.create(
				"localhost", 26010);
		javax.jms.QueueConnectionFactory qcf = QueueTcpConnectionFactory
				.create("localhost", 26010);

		Context jndiCtx = new InitialContext();
		jndiCtx.bind("cf", cf);
		jndiCtx.bind("qcf", qcf);
		jndiCtx.bind(QUEUE_REQUEST, queueRequest);
		jndiCtx.bind(QUEUE_RESPONSE, queueResponse);
		jndiCtx.close();
		System.out.println("   (Destinations bound)");

		AdminModule.disconnect();
		System.out.println("   (disconnect from JORAM server)");
		System.out.println("JMS destinations initialized...");

	}

	protected static void startServer() throws Exception {
		System.out.println("Start JORAM server S0...");
		
		AgentServer.init(new String[] { "0", "./joram0" });
		AgentServer.start();
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				stopServer();
			}
		});

		System.out.println("...JORAM server S0 started");
	}

	protected static void stopServer() {
		System.out.println("Stop JORAM server S0");
		AgentServer.stop();
	}

}
