/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.travelagency.hotel.jms;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;
import javax.naming.NamingException;
import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.objectweb.petals.demo.travelagency.hotel.HotelBookingService;
import org.objectweb.petals.demo.travelagency.hotel.HotelBookingServiceImpl;
import org.objectweb.petals.demo.travelagency.hotel.beans.HotelBookingRequestBean;
import org.objectweb.petals.demo.travelagency.hotel.beans.HotelBookingResultBean;
import org.objectweb.petals.demo.travelagency.hotel.gui.Console;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class MsgListener implements MessageListener {
    
    
    private Console console;

    /**
     * Creates a new instance of {@link MsgListener}
     * 
     */
    public MsgListener() {
    }

    /**
     * Creates a new instance of {@link MsgListener}
     * 
     * @param ident
     */
    public MsgListener(String ident, Console console) {
        this.console = console;
        console.logMessage("MsgListener ready to listen for " + ident);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jms.MessageListener#onMessage(javax.jms.Message)
     */
    public void onMessage(Message msg) {
        console.logMessage("Getting a new hotel booking request...");

        JOptionPane.showMessageDialog(console,
            "New hotel booking request received - Click to continue");

        try {
            if (msg instanceof TextMessage) {

                // get info from incoming message
                HotelBookingService bookingService = new HotelBookingServiceImpl();
                String soapIn = ((TextMessage) msg).getText();

                System.out.println(soapIn);

                HotelBookingRequestBean request = retrieveRequest(soapIn);
                HotelBookingResultBean booking = bookingService
                    .bookHotel(request);

                // send result
                Sender sender = new Sender(console);
                sender.send(booking);

            } else {
                console.logMessage("Error : unrecognized message type");
            }
        } catch (JMSException e) {
            System.err.println("Exception in listener: " + e);
        } catch (NamingException e) {
            System.err.println("Exception in listener: " + e);
        } catch (Exception e) {
            System.err.println("Exception in listener: " + e);
        }
    }

    /**
     * Parse the incoming request
     * 
     * @param soapString
     * @return
     */
    private HotelBookingRequestBean retrieveRequest(String soapString)
        throws Exception {
        String firstName = "";
        String lastName = "";
        String checkIn = "";
        String checkOut = "";
        String city = "";

        Document doc = null;
        doc = parseXml(soapString);
        // get the request node
        Node requestValues = doc.getDocumentElement().getFirstChild();

        List<Node> values = getNodeChildren(requestValues);
        for (Node node : values) {
            if ("firstName".equalsIgnoreCase(node.getLocalName())) {
                firstName = node.getTextContent();
            } else if ("lastName".equalsIgnoreCase(node.getLocalName())) {
                lastName = node.getTextContent();
            } else if ("checkIn".equalsIgnoreCase(node.getLocalName())) {
                checkIn = node.getTextContent();
            } else if ("checkOut".equalsIgnoreCase(node.getLocalName())) {
                checkOut = node.getTextContent();
            } else if ("city".equalsIgnoreCase(node.getLocalName())) {
                city = node.getTextContent();
            }
        }
        return new HotelBookingRequestBean(firstName, lastName, city, checkIn,
            checkOut);
    }

    /**
     * Create a document from the xml string
     * 
     * @param xmlString
     * @return
     */
    private Document parseXml(final String xmlString) throws Exception {

        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
            .newInstance();
        docBuilderFactory.setNamespaceAware(true);

        DocumentBuilder docBuilder = null;
        Document document = null;

        try {
            docBuilder = docBuilderFactory.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            throw new Exception("Bad XML parser configuration", pce);
        }

        try {
            document = docBuilder.parse(new ByteArrayInputStream(xmlString
                .getBytes()));
        } catch (SAXException saxe) {
            throw new Exception("JBI descriptor is not well formed XML", saxe);
        } catch (IOException ioe) {
            throw new Exception("Can't read JBI descriptor", ioe);
        }
        return document;
    }
    
    /**
	 * A List of Node that contains all children of this node; nodes must be of
	 * type Node.ELEMENT_NODE to be returned. If there are no children, this is
	 * a NodeList containing no nodes.
	 *
	 * @param node
	 *            the node from which children will be extracted
	 * @return a List of child Node of type Node.ELEMENT_NODE
	 */
	public static List<Node> getNodeChildren(final Node node) {
		List<Node> children = new ArrayList<Node>();
		if (node != null) {
			NodeList childrenList = node.getChildNodes();
			for (int i = 0; i < childrenList.getLength(); i++) {
				Node child = childrenList.item(i);
				if (child.getNodeType() == Node.ELEMENT_NODE) {
					children.add(child);
				}
			}
		}

		return children;
	}


}
