/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.travelagency.hotel.jms;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.petals.demo.travelagency.hotel.beans.HotelBookingResultBean;
import org.objectweb.petals.demo.travelagency.hotel.gui.Console;

/**
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class Sender {
    
    private Console console;

    static Context ictx = null;

    /**
     * Creates a new instance of {@link Sender}
     * 
     * @param hotelBookingResultBean
     */
    public Sender(Console console) {
        this.console = console;
    }

    /**
     * Send the result on the queue
     * 
     * @param hotelBookingResultBean
     * @throws JMSException
     * @throws NamingException
     */
    public void send(HotelBookingResultBean hotelBookingResultBean)
        throws JMSException, NamingException {
        console.logMessage("Sending message to travel agency...");

        ictx = new InitialContext();
        Queue queueResponse = (Queue) ictx.lookup(ClassicAdmin.QUEUE_RESPONSE);
        QueueConnectionFactory qcf = (QueueConnectionFactory) ictx
            .lookup("qcf");
        ictx.close();

        QueueConnection qc = qcf.createQueueConnection();
        QueueSession qs = qc.createQueueSession(true, 0);
        QueueSender qsend = qs.createSender(queueResponse);
        TextMessage msg = qs.createTextMessage();

        msg.setText(getBookingAsXml(hotelBookingResultBean));
        msg.setStringProperty("javax.jbi.messaging.operation", "setHotelBookingResponse");
        qsend.send(msg);

        qs.commit();
        qc.close();
    }

    /**
     * 
     * @param hotelBookingResultBean
     * @return
     */
    private String getBookingAsXml(HotelBookingResultBean hotelBookingResultBean) {
        StringBuffer buffer = new StringBuffer();
        buffer.append("<bookHotelResponse>");
        buffer.append("<return><hotelInfo><hotelName>").append(
            hotelBookingResultBean.getHotelInfo().getHotelName()).append(
            "</hotelName>");
        buffer.append("<phoneNumber>").append(
            hotelBookingResultBean.getHotelInfo().getPhoneNumber()).append(
            "</phoneNumber>");
        buffer.append("<address1>").append(
            hotelBookingResultBean.getHotelInfo().getAddress1()).append(
            "</address1>");
        buffer.append("<address2>").append(
            hotelBookingResultBean.getHotelInfo().getAddress2()).append(
            "</address2>");
        buffer.append("<city>").append(
            hotelBookingResultBean.getHotelInfo().getCity()).append("</city>");
        buffer.append("<nightPrice>").append(
            hotelBookingResultBean.getHotelInfo().getNightPrice()).append(
            "</nightPrice>");
        buffer.append("<description>").append(
            hotelBookingResultBean.getHotelInfo().getDescription()).append(
            "</description>");
        buffer.append("<rating>").append(
            hotelBookingResultBean.getHotelInfo().getRating()).append(
            "</rating></hotelInfo>");
        buffer.append("<firstName>").append(
            hotelBookingResultBean.getFirstName()).append("</firstName>");
        buffer.append("<lastName>")
            .append(hotelBookingResultBean.getLastName()).append("</lastName>");
        buffer.append("<checkIn>").append(hotelBookingResultBean.getCheckIn())
            .append("</checkIn>");
        buffer.append("<checkOut>")
            .append(hotelBookingResultBean.getCheckOut()).append("</checkOut>");
        buffer.append("<nbNights>")
            .append(hotelBookingResultBean.getNbNights()).append("</nbNights>");
        buffer.append("<price>").append(hotelBookingResultBean.getPrice())
            .append("</price>");
        buffer.append("</return></bookHotelResponse>");
        return buffer.toString();
    }
}
