/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.travelagency.web;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.soap.SOAP11Constants;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;

/**
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class AxisBookingClient implements BookingClient {

    protected Logger logger = Logger.getLogger(this.getClass().toString());

    private EndpointReference targetEPR;

    private static String operation = "orchestrate";
    
    /**
     * Creates a new instance of {@link AxisBookingClient}
     *
     * @param endpoint
     */
    public AxisBookingClient(String endpoint) {
        targetEPR = new EndpointReference(endpoint);
    }

    /*
     * (non-Javadoc)
     * @see org.objectweb.petals.demo.travelagency.web.BookingClient#submit(org.objectweb.petals.demo.travelagency.web.BookingInfoBean)
     */
    public void submit(BookingInfoBean bean) throws AxisFault {
        logger.log(Level.INFO, "Submitting request to travel agency");
        OMElement payload = createPayload(bean);
        Options options = createOptions();
        ServiceClient client = new ServiceClient();
        client.setOptions(options);
        client.fireAndForget(payload);
    }

    /**
     * 
     * @param _firstName
     * @param _lastName
     * @param _address
     * @param _salary
     * @param _propertyTaxes
     * @param _insurrance
     * @param _autoPayment
     * @param _creditCards
     * @param _otherPayments
     * @return
     */
    private OMElement createPayload(BookingInfoBean bean) {
        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMNamespace omNs = fac.createOMNamespace(
            "http://petals.objectweb.org/", getServiceName());
        OMElement method = fac.createOMElement(operation, omNs);
        OMElement value = fac.createOMElement("param0", omNs);

        OMElement firstName = fac.createOMElement("first_name", omNs);
        firstName.addChild(fac.createOMText(value, bean.getFirstName()));
        value.addChild(firstName);

        OMElement lastName = fac.createOMElement("last_name", omNs);
        lastName.addChild(fac.createOMText(value, bean.getLastName()));
        value.addChild(lastName);

        OMElement email = fac.createOMElement("email", omNs);
        email.addChild(fac.createOMText(value, bean.getEmail()));
        value.addChild(email);

        OMElement departureDate = fac.createOMElement("departure_date", omNs);
        departureDate.addChild(fac.createOMText(value, bean.getDepartDate()));
        value.addChild(departureDate);

        OMElement returnDate = fac.createOMElement("return_date", omNs);
        returnDate.addChild(fac.createOMText(value, bean.getReturnDate()));
        value.addChild(returnDate);

        OMElement fromCity = fac.createOMElement("from_city", omNs);
        fromCity.addChild(fac.createOMText(value, bean.getFromCity()));
        value.addChild(fromCity);

        OMElement toCity = fac.createOMElement("to_city", omNs);
        toCity.addChild(fac.createOMText(value, bean.getToCity()));
        value.addChild(toCity);

        method.addChild(value);

        return method;
    }

    /**
     * Get the service name from the target EPR
     * 
     * @return
     */
    private String getServiceName() {
        String eprName = targetEPR.getAddress();
        return eprName
            .substring(eprName.lastIndexOf("/") + 1, eprName.length());
    }

    /**
     * Create the Axis2 options
     * 
     * @return
     */
    protected Options createOptions() {
        Options options = new Options();

        // set destination address
        options.setTo(targetEPR);

        // set operation
        options.setAction(operation);
        options.setSoapVersionURI(SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI);
        options.setTransportInProtocol(Constants.TRANSPORT_HTTP);

        // FIXME
        options.setProperty(
            org.apache.axis2.Constants.Configuration.CHARACTER_SET_ENCODING,
            "UTF-8");
        options.setProperty(
            org.apache.axis2.transport.http.HTTPConstants.CHUNKED,
            org.apache.axis2.Constants.VALUE_FALSE);
        options.setProperty(
            AddressingConstants.DISABLE_ADDRESSING_FOR_OUT_MESSAGES,
            Boolean.TRUE);
        return options;
    }
}
