/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.travelagency.web;

import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.axis2.AxisFault;

/**
 * A servlet for handling travel agency booking queries
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class BookingServlet extends HttpServlet {

    private String travelAgencyService;

    private static final String TRAVELAGENCY_SERVICE = "travelagency-service";

    private static final long serialVersionUID = 1L;

    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.GenericServlet#init(javax.servlet.ServletConfig)
     */
    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        travelAgencyService = config.getInitParameter(TRAVELAGENCY_SERVICE);
        if (travelAgencyService == null)
            throw new ServletException(
                "Unable to get endpoint address from web.xml file");
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException {
        this.doPost(req, resp);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException {

        String firstName = req.getParameter("firstname");
        String lastName = req.getParameter("lastname");
        String email = req.getParameter("email");
        String departDate = req.getParameter("depart");
        String returnDate = req.getParameter("return");
        String from = req.getParameter("from");
        String to = req.getParameter("to");

        BookingInfoBean bookingBean = new BookingInfoBean(firstName, lastName,
            email, departDate, returnDate, from, to);

        AxisBookingClient client = new AxisBookingClient(travelAgencyService);
        // create WS client and call service
        try {
            client.submit(bookingBean);
        } catch (AxisFault e) {
            writeError(resp.getOutputStream(), e);
        }
        
        writeMessage(bookingBean, resp.getOutputStream());
    }

    /**
     * Write the error to the output stream
     * 
     * @param out
     * @param e
     * @throws IOException
     */
    private void writeError(OutputStream out, AxisFault e) throws IOException {
        StringBuffer sb = new StringBuffer("An error occurend during WS call:<br>");
        sb.append(e.getMessage());
        out.write(sb.toString().getBytes());
    }
    
    /**
     * Write a message for the user
     * 
     * @param bean
     * @param out
     * @throws IOException
     */
    private void writeMessage(BookingInfoBean bean, OutputStream out) throws IOException {
        StringBuffer sb = new StringBuffer("<html><body><br><br><center>");
        sb.append("Your booking request has been sent to the PEtALS travel agency, you will soon receive a confimation email at ");
        sb.append("<a href='mailto:");
        sb.append(bean.getEmail());
        sb.append("'>");
        sb.append(bean.getEmail());
        sb.append("</a>");
        sb.append("<br><br>");
        sb.append("<a href=\"index.jsp\">- return -</a>");
        sb.append("</center></body></html>");
        out.write(sb.toString().getBytes());
    }
}
