/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.travelagency.workflow;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.swing.JOptionPane;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.demo.travelagency.gui.Console;
import org.objectweb.petals.demo.travelagency.workflow.bo.AirlineBookingResponse;
import org.objectweb.petals.demo.travelagency.workflow.bo.AirlineBookingResponseTrip;
import org.objectweb.petals.demo.travelagency.workflow.bo.HotelBookingResponse;
import org.objectweb.petals.demo.travelagency.workflow.bo.HotelInfoBean;
import org.objectweb.petals.demo.travelagency.workflow.bo.TravelAgencyRequest;
import org.objectweb.petals.tools.jbicommon.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * 
 * @author ofabre
 * 
 */
public class TravelAgencyWorkflowImpl implements TravelAgencyWorkflow {

    private Logger logger;

    private DeliveryChannel channel;

    private HotelBookingResponse hotelBookingResponse;

    public final static String AIRLINE_SERVICE = "AirlineBookingService";

    public final static String HOTEL_SERVICE = "BookingHotelRequestService";

    public final static String AIRLINE_SERVICE_OP = "bookFlight";

    public final static String HOTEL_SERVICE_OP = "bookHotel";

    public final static String SUMMARY_SERVICE = "sendBookingSummaryService";

    public final static String DEFAULT_NS = "http://petals.objectweb.org/";

    private Semaphore semaphore;

    private Console console;

    public TravelAgencyWorkflowImpl(Logger logger, DeliveryChannel channel) {
        super();
        this.logger = logger;
        this.channel = channel;
        console = new Console();
        console.setVisible(true);
        semaphore = new Semaphore();
    }

    public void orchestrate(String content) throws Exception {
        /*
         * retrieve request parameters
         */
        TravelAgencyRequest request = retrieveInitialRequestParameters(content);
        logRequest(request);
        JOptionPane.showMessageDialog(console,
            "Travel Agency receives a request");

        /*
         * execute workflow to process
         */
        console.getJTextArea().append(
            "###### PROCESSING TRAVEL AGENCY ORCHESTRATION\n");

        /*
         * First step : airline booking
         */
        console.getJTextArea().append("###### First step : Air line booking\n");
        AirlineBookingResponse airlineBookingResponse = bookAirline(request);
        logAirlineResponse(airlineBookingResponse);
        JOptionPane.showMessageDialog(console,
            "Response received from Airline service");

        /*
         * Second step : hotel booking
         */
        console.getJTextArea().append("###### Second Step : Hotel Booking\n");
        bookHotel(request);
        semaphore.acquire();
        logHotelResponse();
        JOptionPane.showMessageDialog(console,
            "Response received from Hotel service");

        /*
         * Last step : summary
         */
        console.getJTextArea().append("###### Last Step : Send a summary\n");
        doSendSummary(request, airlineBookingResponse);
        console.getJTextArea().append(
            "###### Travel Agency Workflow finalized\n");
        this.hotelBookingResponse = null;

    }

    private void logHotelResponse() {
        console.getJTextArea().append("Hotel Response Information : \n");
        console.getJTextArea().append("\tHotel Information : \n");
        HotelInfoBean hotelInfo = hotelBookingResponse.getHotelInfo();
        console.getJTextArea().append(
            "\t\tName : " + hotelInfo.getHotelName() + "\n");
        console.getJTextArea().append(
            "\t\tPhone Number : " + hotelInfo.getPhoneNumber() + "\n");
        console.getJTextArea().append(
            "\t\tAddress : " + hotelInfo.getAddress1() + " "
                + hotelInfo.getCity() + "\n");
        console.getJTextArea().append(
            "\t\tDescription : " + hotelInfo.getDescription() + "\n");
        console.getJTextArea().append(
            "\t\tRating : " + hotelInfo.getRating() + "\n");
        console.getJTextArea().append(
            "\tCheck In : " + hotelBookingResponse.getCheckIn() + "\n");
        console.getJTextArea().append(
            "\tCheck Out : " + hotelBookingResponse.getCheckOut() + "\n");
        console.getJTextArea()
            .append(
                "\tNumber of nights : " + hotelBookingResponse.getNbNights()
                    + "\n");
        console.getJTextArea().append(
            "\tPrice : " + hotelBookingResponse.getPrice() + "\n");

    }

    private void logAirlineResponse(
        AirlineBookingResponse airlineBookingResponse) {
        console.getJTextArea().append("Airline Response Information : \n");
        console.getJTextArea().append("\tDeparture Trip : \n");
        AirlineBookingResponseTrip departure = airlineBookingResponse
            .getDepartureTrip();
        console.getJTextArea().append(
            "\t\tFrom : " + departure.getFromCity() + "\n");
        console.getJTextArea().append(
            "\t\tTo : " + departure.getToCity() + "\n");
        console.getJTextArea().append(
            "\t\tDeparture : " + departure.getDate() + " at "
                + departure.getDepartureHour() + "\n");
        console.getJTextArea().append(
            "\t\tArrival : " + departure.getDate() + " at "
                + departure.getArrivalHour() + "\n");
        console.getJTextArea().append("\tReturn Trip : \n");
        AirlineBookingResponseTrip returnTrip = airlineBookingResponse
            .getReturnTrip();
        console.getJTextArea().append(
            "\t\tFrom : " + returnTrip.getFromCity() + "\n");
        console.getJTextArea().append(
            "\t\tTo : " + returnTrip.getToCity() + "\n");
        console.getJTextArea().append(
            "\t\tDeparture : " + returnTrip.getDate() + " at "
                + returnTrip.getDepartureHour() + "\n");
        console.getJTextArea().append(
            "\t\tArrival : " + returnTrip.getDate() + " at "
                + returnTrip.getArrivalHour() + "\n");
    }

    private void logRequest(TravelAgencyRequest request) {
        console.getJTextArea().append("Client Request Information : \n");
        console.getJTextArea().append(
            "\tFirst Name : " + request.getFirstName() + "\n");
        console.getJTextArea().append(
            "\tLast Name : " + request.getLastName() + "\n");
        console.getJTextArea()
            .append("\tE-mail : " + request.getEmail() + "\n");
        console.getJTextArea().append(
            "\tFrom : " + request.getFromCity() + "\n");
        console.getJTextArea().append("\tTo : " + request.getToCity() + "\n");
        console.getJTextArea().append(
            "\tDeparture Date : " + request.getDepartureDate() + "\n");
        console.getJTextArea().append(
            "\tReturn Date : " + request.getReturnDate() + "\n");
    }

    private void doSendSummary(TravelAgencyRequest request,
        AirlineBookingResponse airlineBookingResponse)
        throws PEtALSComponentSDKException, MessagingException {

        /*
         * Create the summary
         */
        Source summary = createSummary(request, airlineBookingResponse);

        /*
         * Create a new message exchange for SendSummaryService
         */
        MessageExchange msg = channel.createExchangeFactory()
            .createInOnlyExchange();
        msg.setService(new QName(DEFAULT_NS, SUMMARY_SERVICE));

        /*
         * Create the inMessage to send
         */
        NormalizedMessage inNM = msg.createMessage();
        inNM.setContent(summary);
        ((InOnly) msg).setInMessage(inNM);

        /*
         * Send the message exchange
         */
        channel.sendSync(msg);

        /*
         * Check if summary has been correctly sent
         */
        if (!ExchangeStatus.DONE.equals(msg.getStatus())) {
            logger.warning("Summary isn't correctly sent !");
        }

    }

    private Source createSummary(TravelAgencyRequest request,
        AirlineBookingResponse airlineBookingResponse)
        throws PEtALSComponentSDKException {

        StringBuffer summary = new StringBuffer();
        summary.append("<summary>\n");
        summary.append("Client Information : \n");
        summary.append("\tFirst Name : " + request.getFirstName() + "\n");
        summary.append("\tLast Name : " + request.getLastName() + "\n");
        summary.append("\tE-Mail : " + request.getEmail() + "\n");
        summary.append(" \n");
        summary.append("Airline Booking Information : \n");
        summary.append("\tDeparture Trip : \n");
        AirlineBookingResponseTrip departure = airlineBookingResponse
            .getDepartureTrip();
        summary.append("\t\tFrom : " + departure.getFromCity() + "\n");
        summary.append("\t\tTo : " + departure.getToCity() + "\n");
        summary.append("\t\tDeparture : " + departure.getDate() + " at "
            + departure.getDepartureHour() + "\n");
        summary.append("\t\tArrival : " + departure.getDate() + " at "
            + departure.getArrivalHour() + "\n");
        summary.append("\tReturn Trip : \n");
        AirlineBookingResponseTrip returnTrip = airlineBookingResponse
            .getReturnTrip();
        summary.append("\t\tFrom : " + returnTrip.getFromCity() + "\n");
        summary.append("\t\tTo : " + returnTrip.getToCity() + "\n");
        summary.append("\t\tDeparture : " + returnTrip.getDate() + " at "
            + returnTrip.getDepartureHour() + "\n");
        summary.append("\t\tArrival : " + returnTrip.getDate() + " at "
            + returnTrip.getArrivalHour() + "\n");
        summary.append(" \n");
        summary.append("Hotel Booking Information : \n");
        summary.append("\tHotel Information : \n");
        HotelInfoBean hotelInfo = hotelBookingResponse.getHotelInfo();
        summary.append("\t\tName : " + hotelInfo.getHotelName() + "\n");
        summary.append("\t\tPhone Number : " + hotelInfo.getPhoneNumber()
            + "\n");
        summary.append("\t\tAddress : " + hotelInfo.getAddress1() + " "
            + hotelInfo.getCity() + "\n");
        summary
            .append("\t\tDescription : " + hotelInfo.getDescription() + "\n");
        summary.append("\t\tRating : " + hotelInfo.getRating() + "\n");
        summary.append("\tCheck In : " + hotelBookingResponse.getCheckIn()
            + "\n");
        summary.append("\tCheck Out : " + hotelBookingResponse.getCheckOut()
            + "\n");
        summary.append("\tNumber of nights : "
            + hotelBookingResponse.getNbNights() + "\n");
        summary.append("\tPrice : " + hotelBookingResponse.getPrice() + "\n");
        summary.append("</summary>");

        return SourceHelper.createSource(summary.toString());
    }

    private void bookHotel(TravelAgencyRequest request)
        throws MessagingException, PEtALSComponentSDKException {
        Source source = createHotelInMessageContent(request);

        /*
         * Create a new message exchange for AirlineService
         */
        InOnly msg = channel.createExchangeFactory().createInOnlyExchange();
        msg.setService(new QName(DEFAULT_NS, HOTEL_SERVICE));
        msg.setOperation(new QName(HOTEL_SERVICE_OP));

        /*
         * Create the inMessage to send
         */
        NormalizedMessage inNM = msg.createMessage();
        inNM.setContent(source);
        msg.setInMessage(inNM);

        /*
         * Send the message exchange
         */
        channel.sendSync(msg);

    }

    private Source createHotelInMessageContent(TravelAgencyRequest request)
        throws PEtALSComponentSDKException {
        StringBuffer soap = new StringBuffer();
        soap.append("<" + HOTEL_SERVICE_OP + ">");
        soap.append("<request>");
        soap.append("<firstName>" + request.getFirstName() + "</firstName>");
        soap.append("<lastName>" + request.getLastName() + "</lastName>");
        soap.append("<city>" + request.getToCity() + "</city>");
        soap.append("<checkIn>" + request.getDepartureDate() + "</checkIn>");
        soap.append("<checkOut>" + request.getReturnDate() + "</checkOut>");
        soap.append("</request>");
        soap.append("</" + HOTEL_SERVICE_OP + ">");

        return SourceHelper.createSource(soap.toString());
    }

    private AirlineBookingResponse bookAirline(TravelAgencyRequest request)
        throws PEtALSComponentSDKException, MessagingException {

        Source source = createAirlineInMessageContent(request);

        /*
         * Create a new message exchange for AirlineService
         */
        InOut msg = channel.createExchangeFactory().createInOutExchange();
        msg.setService(new QName(DEFAULT_NS, AIRLINE_SERVICE));
        msg.setOperation(new QName(AIRLINE_SERVICE_OP));

        /*
         * Create the inMessage to send
         */
        NormalizedMessage inNM = msg.createMessage();
        inNM.setContent(source);
        msg.setInMessage(inNM);

        /*
         * Send the message exchange
         */
        channel.sendSync(msg);

        /*
         * Get response from EvalRate service
         */
        NormalizedMessage outNM = msg.getOutMessage();
        String response = SourceHelper.createString(outNM.getContent());

        AirlineBookingResponse airlineBookingResponse = retrieveAirlineResponseParameters(response);

        return airlineBookingResponse;
    }

    private AirlineBookingResponse retrieveAirlineResponseParameters(
        String response) throws HandlingException {
        String firstName = "";
        String id = "";
        AirlineBookingResponseTrip departureTrip = null;
        AirlineBookingResponseTrip returnTrip = null;

        Document doc = parseXml(response);
        Node responseValues = doc.getDocumentElement().getFirstChild();

        List<Node> values = XMLUtil.getNodeChildren(responseValues);
        for (Node node : values) {
            if ("firstName".equalsIgnoreCase(node.getLocalName())) {
                firstName = node.getTextContent();
            } else if ("id".equalsIgnoreCase(node.getLocalName())) {
                id = node.getTextContent();
            } else if ("departureTrip".equalsIgnoreCase(node.getLocalName())) {
                departureTrip = createResponseTrip(node);
            } else if ("returnTrip".equalsIgnoreCase(node.getLocalName())) {
                returnTrip = createResponseTrip(node);
            }
        }

        return new AirlineBookingResponse(departureTrip, returnTrip, firstName,
            id);
    }

    private AirlineBookingResponseTrip createResponseTrip(Node node) {
        AirlineBookingResponseTrip responseTrip;
        String arrivalHour = "";
        String compagny = "";
        String date = "";
        String departureHour = "";
        String flightNb = "";
        String fromCity = "";
        String price = "";
        String seatNb = "";
        String toCity = "";
        List<Node> tripValues = XMLUtil.getNodeChildren(node);
        for (Node tripNode : tripValues) {
            if ("arrivalHour".equalsIgnoreCase(tripNode.getLocalName())) {
                arrivalHour = tripNode.getTextContent();
            } else if ("compagny".equalsIgnoreCase(tripNode.getLocalName())) {
                compagny = tripNode.getTextContent();
            } else if ("date".equalsIgnoreCase(tripNode.getLocalName())) {
                date = tripNode.getTextContent();
            } else if ("departureHour"
                .equalsIgnoreCase(tripNode.getLocalName())) {
                departureHour = tripNode.getTextContent();
            } else if ("flightNb".equalsIgnoreCase(tripNode.getLocalName())) {
                flightNb = tripNode.getTextContent();
            } else if ("fromCity".equalsIgnoreCase(tripNode.getLocalName())) {
                fromCity = tripNode.getTextContent();
            } else if ("price".equalsIgnoreCase(tripNode.getLocalName())) {
                price = tripNode.getTextContent();
            } else if ("seatNb".equalsIgnoreCase(tripNode.getLocalName())) {
                seatNb = tripNode.getTextContent();
            } else if ("toCity".equalsIgnoreCase(tripNode.getLocalName())) {
                toCity = tripNode.getTextContent();
            }
        }
        responseTrip = new AirlineBookingResponseTrip(arrivalHour, compagny,
            date, departureHour, flightNb, fromCity, price, seatNb, toCity);
        return responseTrip;
    }

    private Source createAirlineInMessageContent(TravelAgencyRequest request)
        throws PEtALSComponentSDKException {
        StringBuffer soap = new StringBuffer();
        soap.append("<ns1:" + AIRLINE_SERVICE_OP
            + " xmlns:ns1=\"http://petals.objectweb.org/\">");
        soap.append("<first_name>" + request.getFirstName() + "</first_name>");
        soap.append("<email>" + request.getEmail() + "</email>");
        soap.append("<from_city>" + request.getFromCity() + "</from_city>");
        soap.append("<to_city>" + request.getToCity() + "</to_city>");
        soap.append("<departure_date>" + request.getDepartureDate()
            + "</departure_date>");
        soap.append("<return_date>" + request.getReturnDate()
            + "</return_date>");
        soap.append("</ns1:" + AIRLINE_SERVICE_OP + ">");

        return SourceHelper.createSource(soap.toString());
    }

    /**
     * Build a Request bo from information contained into the String content (a
     * Soap representation of the TravelAgencyRequest).
     * 
     * @param content
     *            the String content (a Soap representation of the
     *            MortgageRequest)
     * @return a MortgageRequest bo. Can't be null.
     * @throws HandlingException
     * @throws Exception
     */
    protected TravelAgencyRequest retrieveInitialRequestParameters(
        String stringContent) throws HandlingException {

        String firstName = "";
        String lastName = "";
        String email = "";
        String departureDate = "";
        String returnDate = "";
        String fromCity = "";
        String toCity = "";

        System.out.println(stringContent);

        Document doc = parseXml(stringContent);
        Node requestValues = doc.getDocumentElement().getFirstChild();

        List<Node> values = XMLUtil.getNodeChildren(requestValues);
        for (Node node : values) {
            if ("first_name".equalsIgnoreCase(node.getLocalName())) {
                firstName = node.getTextContent();
            } else if ("last_name".equalsIgnoreCase(node.getLocalName())) {
                lastName = node.getTextContent();
            } else if ("email".equalsIgnoreCase(node.getLocalName())) {
                email = node.getTextContent();
            } else if ("departure_date".equalsIgnoreCase(node.getLocalName())) {
                departureDate = node.getTextContent();
            } else if ("return_date".equalsIgnoreCase(node.getLocalName())) {
                returnDate = node.getTextContent();
            } else if ("from_city".equalsIgnoreCase(node.getLocalName())) {
                fromCity = node.getTextContent();
            } else if ("to_city".equalsIgnoreCase(node.getLocalName())) {
                toCity = node.getTextContent();
            }
        }

        TravelAgencyRequest request = new TravelAgencyRequest(firstName,
            lastName, email, departureDate, returnDate, fromCity, toCity);

        return request;
    }

    private Document parseXml(final String xmlString) throws HandlingException {

        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
            .newInstance();
        docBuilderFactory.setNamespaceAware(true);

        DocumentBuilder docBuilder = null;

        try {
            docBuilder = docBuilderFactory.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            throw new HandlingException("Bad XML parser configuration", pce);
        }

        Document document = null;

        try {
            document = docBuilder.parse(new ByteArrayInputStream(xmlString
                .getBytes()));
        } catch (SAXException saxe) {
            throw new HandlingException(
                "JBI descriptor is not well formed XML", saxe);
        } catch (IOException ioe) {
            throw new HandlingException("Can't read JBI descriptor", ioe);
        }
        return document;
    }

    public void setHotelBookingResponse(String content) {
        /*
         * retrieve response parameters
         */

        String firstName = "";
        String lastName = "";
        String checkIn = "";
        String checkOut = "";
        String nbNights = "";
        String price = "";
        HotelInfoBean hotelInfo = null;

        System.out.println(content);

        Document doc = null;
        try {
            doc = parseXml(content);
        } catch (HandlingException e) {
            logger.log(Level.SEVERE, "Error parsing hotel booking response", e);
        }
        if (doc != null) {
            Node responseValues = doc.getDocumentElement().getFirstChild();

            List<Node> values = XMLUtil.getNodeChildren(responseValues);
            for (Node node : values) {
                if ("firstName".equalsIgnoreCase(node.getLocalName())) {
                    firstName = node.getTextContent();
                } else if ("lastName".equalsIgnoreCase(node.getLocalName())) {
                    lastName = node.getTextContent();
                } else if ("checkIn".equalsIgnoreCase(node.getLocalName())) {
                    checkIn = node.getTextContent();
                } else if ("checkOut".equalsIgnoreCase(node.getLocalName())) {
                    checkOut = node.getTextContent();
                } else if ("nbNights".equalsIgnoreCase(node.getLocalName())) {
                    nbNights = node.getTextContent();
                } else if ("price".equalsIgnoreCase(node.getLocalName())) {
                    price = node.getTextContent();
                } else if ("hotelInfo".equalsIgnoreCase(node.getLocalName())) {
                    hotelInfo = createHotelInfo(node);
                }
            }

            this.hotelBookingResponse = new HotelBookingResponse(firstName,
                lastName, hotelInfo, checkIn, checkOut, nbNights, price);
            semaphore.release();
        }
    }

    private HotelInfoBean createHotelInfo(Node node) {
        String hotelName = "";
        String phoneNumber = "";
        String address1 = "";
        String address2 = "";
        String city = "";
        String nightPrice = "";
        String description = "";
        String rating = "";

        List<Node> hotelInfoValues = XMLUtil.getNodeChildren(node);
        for (Node hotelInfoNode : hotelInfoValues) {
            if ("hotelName".equalsIgnoreCase(hotelInfoNode.getLocalName())) {
                hotelName = hotelInfoNode.getTextContent();
            } else if ("phoneNumber".equalsIgnoreCase(hotelInfoNode
                .getLocalName())) {
                phoneNumber = hotelInfoNode.getTextContent();
            } else if ("address1"
                .equalsIgnoreCase(hotelInfoNode.getLocalName())) {
                address1 = hotelInfoNode.getTextContent();
            } else if ("address2"
                .equalsIgnoreCase(hotelInfoNode.getLocalName())) {
                address2 = hotelInfoNode.getTextContent();
            } else if ("city".equalsIgnoreCase(hotelInfoNode.getLocalName())) {
                city = hotelInfoNode.getTextContent();
            } else if ("nightPrice".equalsIgnoreCase(hotelInfoNode
                .getLocalName())) {
                nightPrice = hotelInfoNode.getTextContent();
            } else if ("description".equalsIgnoreCase(hotelInfoNode
                .getLocalName())) {
                description = hotelInfoNode.getTextContent();
            } else if ("rating".equalsIgnoreCase(hotelInfoNode.getLocalName())) {
                rating = hotelInfoNode.getTextContent();
            }
        }
        return new HotelInfoBean(hotelName, phoneNumber, address1, address2,
            city, nightPrice, description, rating);
    }

}
