/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceMBean.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment;

import org.objectweb.petals.PetalsLifeCycle;

/**
 * This custom interface add methods used by redeployment tasks of the
 * SystemState service.
 *
 * @author ofabre
 *
 */
public interface DeploymentServiceMBean extends
    javax.jbi.management.DeploymentServiceMBean, PetalsLifeCycle {

    /**
     * Returns the {@link DeploymentTaskFactory} that allows to build all
     * deployment tasks
     *
     * @return the {@link DeploymentTaskFactory}
     */
    DeploymentTaskFactory getDeploymentTaskFactory();

    /**
     * Try to start all service assemblies. If a service assembly can't be
     * started, the starting process isn't aborted. It returns the list of all
     * started service assemblies.
     *
     * @return an array of succesfuly started service assembly names.
     */
    String[] startAllServiceAssemblies();

    /**
     * Try to stop all service assemblies. If a service assembly can't be
     * stoped, the stoping process isn't aborted. It returns the list of all
     * stoped service assemblies.
     *
     * @return an array of succesfuly stoped service assembly names.
     */
    String[] stopAllServiceAssemblies();

    /**
     * Try to shutdown all service assemblies. If a service assembly can't be
     * shutdown, the shutdowning process isn't aborted. It returns the list of
     * all shutdown service assemblies.
     *
     * @return an array of succesfuly shutdown service assembly names.
     */
    String[] shutdownAllServiceAssemblies();

    /**
     * Try to undeploy all service assemblies. If a service assembly can't be
     * undeployed, the undeployning process isn't aborted. It returns the list
     * of all undeployed service assemblies.
     *
     * @return an array of succesfuly undeployed service assembly names.
     */
    String[] undeployAllServiceAssemblies();

    /**
     * Try to undeploy all service assemblies. If the SA is in the STARTED
     * state, it will be stopped and shutdown. If the SA is in the STOPPED
     * state, it will be shutdown. If a service assembly can't be undeployed,
     * the undeploying process isn't aborted. It returns the list of all
     * undeployed service assemblies.
     *
     * @return an array of succesfuly undeployed service assembly names.
     */
    String[] forceUndeployAllServiceAssemblies();

    /**
     * Try to undeploy a service assembly. If the SA is in the STARTED state, it
     * will be stopped and shutdown. If the SA is in the STOPPED state, it will
     * be shutdown.
     *
     * @return true if the SA is undeployed, false otherwise.
     */
    boolean forceUndeployServiceAssembly(String name);

    /**
     * Get all the service unit names for the given service assembly
     * @param saName
     * @return
     */
    String[] getServiceUnitForServiceAssembly(String saName) throws Exception;
}
