/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: Router.java,v 1.2 2006/03/17 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.routing;

import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;

/**
 * Routes messages from the <code>DeliveryChannel</code> to their destinations.
 * Performs an itinerary resolution, adding for example a transformation service before
 * delivering the message to its recipient, and perform an address resolution,
 * finding the endpoint satisfying the the destination information (finding an
 * endpoint for a specified service). Resolutions are based on defined rules.
 * 
 * Routes messages pushed from the <code>Transporter</code>
 * to the correct <code>DeliveryChannel</code>
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author rnaudin - EBM WebSourcing
 */
public interface Router {

    /**
     * Send the message into the NMR.
     * 
     * @param source
     * @param exchange
     * @param timeOut timeout of the message (in ms). 0 for no timeout
     * @throws RoutingException
     */
    void send(ComponentContextImpl source, MessageExchangeImpl exchange,
        long timeOut) throws RoutingException;

    /**
     * Try to receive a message from MNR if there is any and
     * return it.
     *
     * @param source
     *              The source component context 
     * @param  timeoutMS
     *              The timeout to receive the incomming message
     * @param  block
     *              If <code>True</code>, the pol is blocking until a response                                   
     
     * @returns The message exchange retrieve from the <code>Transporter</code>
     
     * @throws RoutingException
     */
    MessageExchangeImpl receive(ComponentContextImpl source ,long timeoutMS, boolean block) throws RoutingException;
    
    /**
     * Push a message from the MNR to the <code>DeliveryChannel</code>.
     * 
     * @param  exchange
     *              The message exchange to push
     *
     * @throws RoutingException
     */
    void pushResponse(MessageExchangeImpl exchange) throws RoutingException;
}
