/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: LauncherTest.java 11:45:12 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.launch;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.NameController;

/**
 * Test of the launcher
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class LauncherTest extends TestCase {

    
    public void testGetComponentByName() throws NoSuchInterfaceException {
        ContentController contentController = EasyMock
            .createMock(ContentController.class);
        Component component = EasyMock.createMock(Component.class);
        NameController nameController = EasyMock
            .createMock(NameController.class);

        Component[] compos = new Component[] {component};

        EasyMock.expect(contentController.getFcSubComponents()).andReturn(
            compos).anyTimes();
        EasyMock.expect(component.getFcInterface("name-controller")).andReturn(
            nameController).anyTimes();
        EasyMock.expect(nameController.getFcName()).andReturn("test")
            .anyTimes();

        EasyMock.replay(contentController);
        EasyMock.replay(component);
        EasyMock.replay(nameController);

        assertEquals(component, FractalHelper.getComponentByName(contentController,
            "test"));
    }

    public void testGetComponentByNameException()
        throws NoSuchInterfaceException {
        ContentController contentController = EasyMock
            .createMock(ContentController.class);
        Component component = EasyMock.createMock(Component.class);

        Component[] compos = new Component[] {component};

        EasyMock.expect(contentController.getFcSubComponents()).andReturn(
            compos).anyTimes();
        EasyMock.expect(component.getFcInterface("name-controller")).andThrow(
            new NoSuchInterfaceException("name-controller"));

        EasyMock.replay(contentController);
        EasyMock.replay(component);

        assertNull(FractalHelper.getComponentByName(contentController, "test"));
    }
}
