/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ExchangesList.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.Object;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

/**
 * List of MessageExchange objects
 * 
 * @author msauvage
 * 
 */
public class ExchangesList extends ArrayList<MessageExchange> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3728863776793190333L;

	/**
	 * create an empty new ExchangesList
	 * 
	 */
	public ExchangesList() {
	}

	/**
	 * Creates a new ExchangesList based on bulk datas
	 * 
	 * @param bulkDatas
	 *            datas to transform
	 */
	public ExchangesList(List<List<Map<String, Object>>> bulkDatas) {
		for (List<Map<String, Object>> exchange : bulkDatas) {
			this.add(new MessageExchange(exchange));
		}
	}

	public ExchangesList getByComponent(String compoName) {
		ExchangesList sublist = new ExchangesList();
		for (MessageExchange exchange : this) {
			if (exchange.hasMsgForComponent(compoName)) {
				sublist.add(exchange);
			}
		}
		return sublist;
	}

	/**
	 * Returns messages from all exchanges with timestamp corresponding to the
	 * second of a given timestamp
	 * 
	 * @param timestamp
	 *            timestamp of the second wanted
	 * @return List of Message objects
	 */
	public List<Message> getMessagesByTime(long timestamp) {
		List<Message> messages = new ArrayList<Message>();
		ListIterator<MessageExchange> parcExchs = this.listIterator();
		while (parcExchs.hasNext()) {
			MessageExchange currentExch = parcExchs.next();
			messages.addAll(currentExch.getByTime(timestamp));
		}
		return messages;
	}

	/**
	 * Returns number of messages from all exchanges with timestamp
	 * corresponding to the second of a given timestamp
	 * 
	 * @param timestamp
	 *            timestamp of the second wanted
	 * @return number of messages
	 */
	public long getNbMsgByTime(long timestamp) {
		HashMap<String,String> messagesSelection = new HashMap<String,String>();
		//for each Exchange
		for (MessageExchange currentExch : this) {
			//getting complete messages list for this timestamp
			List<Message> msgList = currentExch.getByTime(timestamp);
			for (Message currentMsg : msgList) {
				//creating the key
				String key = currentMsg.getId()+"_"+currentMsg.getType();
				//searching in the selection list a message with same ID
				String searchedType = messagesSelection.get(key);
				//if there is no message with this ID or if the message found
				//has a different type -> adding message to the list
				if(searchedType==null || !searchedType.equals(currentMsg.getType())) {
					messagesSelection.put(key, currentMsg.getType());
				}
			}
		}
		//returning the size of the messages selection
		return messagesSelection.size();
	}

	/**
	 * Returns the number of messages concerning a given component
	 * 
	 * @param compoName
	 *            the name of the component
	 * @return a number of messages as a long
	 */
	public long getNbMsgByComponent(String compoName) {
		long nbMsg = 0;
		for (MessageExchange exchange : this) {
			nbMsg += exchange.getNbMsgByComponent(compoName);
		}
		return nbMsg;
	}

	/**
	 * Return the Exchange with the given id.
	 * 
	 * @param id
	 *            the id of the exchange wanted
	 * @return a MessageExchange, null if Exchange is not found
	 */
	public MessageExchange getExchange(String id) {
		for (MessageExchange exchange : this) {
			if (exchange.getId().equals(id)) {
				return exchange;
			}
		}
		return null;
	}
	
	public boolean hasExchange(String id) {
		for (MessageExchange exchange : this) {
			if (exchange.getId().equals(id)) {
				return true;
			}
		}
		return false;
	}
}
