/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Message.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.Object;

import java.util.HashMap;
import java.util.Map;

/**
 * represents a message : HashMap containings time, component, role, type,
 * status
 */
public class Message extends HashMap<String, Object> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1702374180586290279L;

	/**
	 * Creates a new Message with a HashMap containing base informations
	 * 
	 * @param hashMessage
	 *            Map representation of the message
	 */
	public Message(Map<String, Object> hashMessage) {
		this.put("id", hashMessage.get("id"));
		this.put("time", hashMessage.get("time"));
		this.put("component", hashMessage.get("component"));
		this.put("role", hashMessage.get("role"));
		this.put("type", hashMessage.get("type"));
		this.put("status", hashMessage.get("status"));
	}

	/**
	 * Returns the id of the message
	 * 
	 * @return the id as a string
	 */
	public String getId() {
		return this.get("id").toString();
	}

	/**
	 * Sets the message's id
	 * 
	 * @param id
	 *            the message id
	 */

	public void setId(String id) {
		this.put("id", id);
	}

	/**
	 * Returns the timestamp of the message
	 * 
	 * @return a timestamp
	 */
	public long getTime() {
		return new Long(this.get("time").toString()).longValue();
	}

	/**
	 * Sets the message's time
	 * 
	 * @param time
	 *            a timestamp
	 */
	public void setTime(long time) {
		this.put("time", time);
	}

	/**
	 * Returns the component name of the message
	 * 
	 * @return the component name as a String
	 */
	public String getComponent() {
		return this.get("component").toString();
	}

	/**
	 * Sets the component's name
	 * 
	 * @param component
	 *            the string that is to be the component's name
	 */
	public void setComponent(String component) {
		this.put("component", component);
	}

	/**
	 * Returns the role of the message
	 * 
	 * @return the role as a string
	 */
	public String getRole() {
		return this.get("role").toString();
	}

	/**
	 * Sets the message's role
	 * 
	 * @param role
	 *            the string that is to be the message's role
	 */
	public void setRole(String role) {
		this.put("role", role);
	}

	/**
	 * Returns the type of the message
	 * 
	 * @return the type as a string
	 */
	public String getType() {
		if (this.get("type")!=null) {
			return this.get("type").toString();
		}
		else {
			return null;
		}
	}

	/**
	 * Sets the message's type
	 * 
	 * @param type
	 *            the string that is to be the message's type
	 */
	public void setType(String type) {
		this.put("type", type);
	}

	/**
	 * Returns the status of the message
	 * 
	 * @return the status as a string
	 */
	public String getStatus() {
		return this.get("status").toString();
	}

	/**
	 * Sets the message's status
	 * 
	 * @param status
	 *            the string that is to be the message's status
	 */
	public void setStatus(String status) {
		this.put("status", status);
	}

	/**
	 * Returns the message's informations, separated by pipes, following pattern :
	 * id|time|component|role|type|status
	 * 
	 * @return a String representation of the message
	 */
	public String toString() {
		return this.getId() + "|" + this.getTime() + "|" + this.getComponent()
				+ "|" + this.getRole() + "|" + this.getType() + "|"
				+ this.getStatus();
	}
}
