/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessageExchange.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.Object;

import java.util.List;
import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Map;

/**
 * represents an exchange : a list of Message objects
 * 
 * @author msauvage
 * 
 */
public class MessageExchange extends ArrayList<Message> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6892057539895161917L;

	/**
	 * Creates a new MessageExchange with a list of Map
	 * 
	 * @param exchange
	 *            a bulk list representation of an exchange
	 */
	public MessageExchange(List<Map<String, Object>> exchange) {
		ListIterator<Map<String, Object>> parcMsgs = exchange.listIterator();
		// parcourt la liste, crée les objets Message et les ajoute au ME
		while (parcMsgs.hasNext()) {
			this.add(new Message(parcMsgs.next()));
		}
	}

	/**
	 * Get messages from the exchange with timestamp corresponding to the second
	 * of a given timestamp
	 * 
	 * @param timestamp
	 *            timestamp of the second wanted
	 * @return List of Message objects
	 */
	public List<Message> getByTime(long timestamp) {
		List<Message> messages = new ArrayList<Message>();
		for (Message currentMsg : this) {
			if (currentMsg.getTime() / 1000 == timestamp / 1000) {
				messages.add(currentMsg);
			}
		}
		return messages;
	}

	/**
	 * Returns the number of messages in the exchange concerning a component
	 * 
	 * @param compoName
	 *            the name of the component
	 * @return a number of messages as a long
	 */
	public long getNbMsgByComponent(String compoName) {
		long nbMsg = 0;
		for (Message message : this) {
			if (message.getComponent().equals(compoName)) {
				nbMsg++;
			}
		}
		return nbMsg;
	}

	/**
	 * Indicates if the MessageExchange has messages concerning a component
	 * 
	 * @param compoName
	 *            the component searched name
	 * @return true if MessageExchanges contains messages for the component,
	 *         false if not.
	 */
	public boolean hasMsgForComponent(String compoName) {
		return getNbMsgByComponent(compoName) > 0;
	}

	/**
	 * Returns the id of the Exchange. Search the id in the first message
	 * contained by the Exchange.
	 * 
	 * @return an id, null if exchange has no message.
	 */
	public String getId() {
		if (this.size() > 0) {
			return this.get(0).getId();
		}
		return null;
	}

}
