/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServerConfiguration.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.Object;

import java.util.HashMap;

import javax.management.MBeanServerConnection;

import org.objectweb.petals.tools.webadmin.service.JMXService;
import org.objectweb.petals.tools.webadmin.util.JMXUtils;

/**
 * Represents the configuration of a server as a HashMap of his properties :
 * host, jmxPort, jmxLogin, jmxPassword and with an attribute : a
 * MBeanServerConnection connection to the server
 */
public class ServerConfiguration extends HashMap<String, String> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7879504327924775052L;

	/**
	 * connection to the server
	 */
	MBeanServerConnection connection;

	public ServerConfiguration() {
	}

	/**
	 * Create a new ServerConfiguration with base connection's informations
	 * 
	 * @param host
	 *            host of the server
	 * @param jmxPort
	 *            JMX port of the server
	 * @param jmxLogin
	 *            JMX login of the server
	 * @param jmxPassword
	 *            JMX password of the server
	 */
	public ServerConfiguration(String host, String jmxPort, String jmxLogin,
			String jmxPassword) {
		this.put("host", host);
		this.put("jmxPort", jmxPort);
		this.put("jmxLogin", jmxLogin);
		this.put("jmxPassword", jmxPassword);
	}

	/**
	 * Create a new ServerConfiguration with base informations
	 * 
	 * @param name
	 *            the name of the server
	 * @param host
	 *            host of the server
	 * @param jmxPort
	 *            JMX port of the server
	 * @param jmxLogin
	 *            JMX login of the server
	 * @param jmxPassword
	 *            JMX password of the server
	 */
	public ServerConfiguration(String name, String host, String jmxPort,
			String jmxLogin, String jmxPassword) {
		this.put("name", name);
		this.put("host", host);
		this.put("jmxPort", jmxPort);
		this.put("jmxLogin", jmxLogin);
		this.put("jmxPassword", jmxPassword);
	}

	/**
	 * Create a new ServerConfiguration with base informations
	 * 
	 * @param name
	 *            the name of the server
	 * @param host
	 *            host of the server
	 * @param jmxPort
	 *            JMX port of the server
	 * @param jmxLogin
	 *            JMX login of the server
	 * @param jmxPassword
	 *            JMX password of the server
	 * @param state
	 *            the state of the server (started, stopped)
	 */
	public ServerConfiguration(String name, String host, String jmxPort,
			String jmxLogin, String jmxPassword, String state) {
		this.put("name", name);
		this.put("host", host);
		this.put("jmxPort", jmxPort);
		this.put("jmxLogin", jmxLogin);
		this.put("jmxPassword", jmxPassword);
		this.put("state", state);
	}

	/**
	 * Create a new ServerConfiguration with complete informations
	 * 
	 * @param name
	 *            the name of the server
	 * @param host
	 *            host of the server
	 * @param jmxPort
	 *            JMX port of the server
	 * @param jmxLogin
	 *            JMX login of the server
	 * @param jmxPassword
	 *            JMX password of the server
	 * @param state
	 *            the state of the server (started, stopped)
	 * @param connection
	 *            a MBeanServerConnection connection to the server
	 */
	public ServerConfiguration(String name, String host, String jmxPort,
			String jmxLogin, String jmxPassword, String state,
			MBeanServerConnection connection) {
		this.put("name", name);
		this.put("host", host);
		this.put("jmxPort", jmxPort);
		this.put("jmxLogin", jmxLogin);
		this.put("jmxPassword", jmxPassword);
		this.put("state", state);
		this.connection = connection;
	}

	/**
	 * Returns the name of the server
	 * 
	 * @return the name of the server as a String
	 */
	public String getName() {
		return this.get("name");
	}

	/**
	 * Sets the name of the server
	 * 
	 * @param name
	 *            a String that is to be the server's name
	 */
	public void setName(String name) {
		this.put("name", name);
	}

	/**
	 * Retrieve the name of the server, and sets it
	 * 
	 * @throws Exception
	 */
	public void setName() throws Exception {
		this.put("name", JMXService.getServerName(this));
	}

	/**
	 * Returns the host of the server
	 * 
	 * @return a String representation fo the host
	 */
	public String getHost() {
		return this.get("host");
	}

	/**
	 * Sets the host of the server
	 * 
	 * @param host
	 *            a String that is to be the host of the server
	 */
	public void setHost(String host) {
		this.put("host", host);
	}

	/**
	 * Returns the JMX port
	 * 
	 * @return a string representation of the port
	 */
	public String getJmxPort() {
		return this.get("jmxPort");
	}

	/**
	 * Sets the JMX port
	 * 
	 * @param jmxPort
	 *            a String that is to be the port
	 */
	public void setJmxPort(String jmxPort) {
		this.put("jmxPort", jmxPort);
	}

	/**
	 * Returns the JMX login name
	 * 
	 * @return a string representation of the JMX login name
	 */
	public String getJmxLogin() {
		return this.get("jmxLogin");
	}

	/**
	 * Sets the JMX login name
	 * 
	 * @param jmxLogin
	 *            a String that is to be the JMX login name
	 */
	public void setJmxLogin(String jmxLogin) {
		this.put("jmxLogin", jmxLogin);
	}

	/**
	 * Returns the JMX user password
	 * 
	 * @return the JMX password as a String
	 */
	public String getJmxPassword() {
		return this.get("jmxPassword");
	}

	/**
	 * Sets the JMX user password
	 * 
	 * @param jmxPassword
	 *            a String that is to be password for jmx access
	 */
	public void setJmxPassword(String jmxPassword) {
		this.put("jmxPassword", jmxPassword);
	}

	/**
	 * Returns the state of the server
	 * 
	 * @return a string representation of the state of the server
	 */
	public String getState() {
		return this.get("state");
	}

	/**
	 * Sets the state of the server
	 * 
	 * @param state
	 *            a String that is to be the state
	 */
	public void setState(String state) {
		this.put("state", state);
	}

	/**
	 * Returns the connection to the server
	 * 
	 * @return a MBeanServerConnection connection
	 */
	public MBeanServerConnection getConnection() {
		return this.connection;
	}

	/**
	 * Sets the connection to the server
	 * 
	 * @param state
	 *            a MBeanServerConnection that is to be the connection
	 */
	public void setConnection(MBeanServerConnection connection) {
		this.connection = connection;
	}

	/**
	 * Retrieve a new connection to the server and sets the connection attribute
	 * 
	 */
	public void setConnection() {
		try {
			this.connection = JMXUtils.retrieveConnector(this)
					.getMBeanServerConnection();
		} catch (Exception e) {
			this.connection = null;
		}
	}

	public String toString() {
		return this.getHost() + ":" + this.getJmxPort() + ":"
				+ this.getJmxLogin() + ":" + this.getJmxPassword();
	}

}
