/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessagesPerServerSecondServlet.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.graph;

import java.awt.Color;
import java.awt.Font;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.data.time.Second;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.objectweb.petals.tools.webadmin.Object.ExchangesList;
import org.objectweb.petals.tools.webadmin.Object.ServerConfiguration;
import org.objectweb.petals.tools.webadmin.controller.MonitorController;

public class MessagesPerServerSecond extends HttpServlet {

	private static final long serialVersionUID = 6965105518145222314L;

	/**
	 * action when receiving a call via "GET" HTTP method
	 */
	@SuppressWarnings("unchecked")
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		OutputStream out = response.getOutputStream();
		try {
			HttpSession session = request.getSession();

			// creating datas set
			TimeSeries timeDatas = new TimeSeries(
					"Messages in the last minute", Second.class);
			TimeSeriesCollection dataset = new TimeSeriesCollection();

			// rounding actual timestamp to second
			long endTime = (new Date().getTime() / 1000) * 1000;
			long startTime = endTime - 59000;

			// getting current server configuration
			ServerConfiguration serverConf = (ServerConfiguration) session
					.getAttribute("currentServerConf");
			// getting the whole exchanges list stored in session
			Map<String, ExchangesList> stored_exchanges = (Map<String, ExchangesList>) session
					.getAttribute("stored_exchanges");
			ExchangesList exchList = stored_exchanges.get(serverConf.getName());
			MonitorController monitorCtrlr = (MonitorController) session
					.getAttribute("MonitorController");

			Calendar cal = Calendar.getInstance();
			for (long timer = startTime; timer <= endTime; timer = timer + 1000) {
				long currentNbMsgs = 0;
				if (exchList!=null)
					currentNbMsgs = monitorCtrlr.getNbMsgByTime(exchList,
						timer);
				// transforming timer in Second object for graph
				Date currentTime = new Date(timer);
				cal.setTime(currentTime);
				timeDatas.add(new Second(currentTime), currentNbMsgs);
			}
			dataset.addSeries(timeDatas);
			String xlabel = "time";
			String ylabel = "Msgs/Sec";
			JFreeChart chart = ChartFactory.createTimeSeriesChart(
					"Messages in the last minute", // title
					xlabel, // x-axis label
					ylabel, // y-axis label
					dataset, // data
					false, // create legend?
					true, // generate tooltips?
					false // generate URLs?
					);
			chart.setBackgroundPaint(new Color(252, 248, 222));
			chart.getTitle().setFont(new Font("sans-serif", Font.BOLD, 12));
			response.setContentType("image/png");
			ChartUtilities.writeChartAsPNG(out, chart, 500, 300);
		} catch (Exception e) {
			e.printStackTrace(System.out);
		} finally {
			if(out!=null) {
				out.flush();
				out.close();
			}
		}
	}

}
