/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessagesRepartitionServlet.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.graph;

import java.awt.Color;
import java.awt.Font;
import java.io.IOException;
import java.io.OutputStream;
import java.text.NumberFormat;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.StandardPieSectionLabelGenerator;
import org.jfree.chart.plot.PiePlot;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.util.Rotation;
import org.jfree.util.SortOrder;
import org.objectweb.petals.tools.webadmin.Object.ExchangesList;
import org.objectweb.petals.tools.webadmin.Object.ServerConfiguration;
import org.objectweb.petals.tools.webadmin.controller.MonitorController;
import org.objectweb.petals.tools.webadmin.service.JMXService;

public class MessagesRepartition extends HttpServlet {

	private static final long serialVersionUID = 3711058136549240746L;

	/**
	 * action when receiving a call via "GET" HTTP method -> creation of the pie
	 * chart for messages repartition by component
	 */
	@SuppressWarnings("unchecked")
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		OutputStream out = response.getOutputStream();
		try {
			HttpSession session = request.getSession();
			// creating datas set
			DefaultPieDataset msgDatas = new DefaultPieDataset();

			// getting current server configuration
			ServerConfiguration serverConf = (ServerConfiguration) session
					.getAttribute("currentServerConf");
			// getting the whole exchanges list stored in session
			ExchangesList exchList = ((Map<String, ExchangesList>) session
					.getAttribute("stored_exchanges"))
					.get(serverConf.getName());
			MonitorController monitorCtrlr = (MonitorController) session
			.getAttribute("MonitorController");

			if (exchList != null && exchList.size() > 0) {
				// retrieving actual active components
				List<String> serverComponents = JMXService.getComponentsName(
						serverConf.getConnection(), "ALL");
				// adding the number of messages for each component to the
				// datas set
				if (serverComponents.size() > 0) {
					for (String component : serverComponents) {
						msgDatas.setValue(component, monitorCtrlr
								.getNbMsgByComponent(exchList,component));
					}
				}
			} else {
				msgDatas.setValue("none", 100);
			}

			// alphabetical sort of components in chart
			msgDatas.sortByKeys(SortOrder.ASCENDING);
			// creation of chart
			JFreeChart chart = ChartFactory.createPieChart(
					"Messages Repartition", // title
					msgDatas, // datas
					false, // legend
					true, // tooltips
					false // urls
					);
			// sets background in yellow to be like transparent in application
			chart.setBackgroundPaint(new Color(252, 248, 222));
			chart.getTitle().setFont(new Font("sans-serif", Font.BOLD, 12));

			PiePlot plot = (PiePlot) chart.getPlot();
			// label formating with pattern "ComponentName - number (percentage
			// %)"
			plot.setLabelGenerator(new StandardPieSectionLabelGenerator(
					"{0} - {1} ({2})", NumberFormat.getNumberInstance(),
					NumberFormat.getPercentInstance()));
			// plots formating
			plot.setStartAngle(210);
			plot.setDirection(Rotation.CLOCKWISE);
			plot.setForegroundAlpha(0.8f);
			plot.setBaseSectionOutlinePaint(Color.BLACK);
			plot.setShadowXOffset(2.0);
			plot.setShadowYOffset(2.0);
			plot.setShadowPaint(new Color(192, 192, 192));
			// exploding plots : 5% each
			List<String> datasKeys = msgDatas.getKeys();
			for (String key : datasKeys) {
				plot.setExplodePercent(key, 0.05);
			}

			// returning chart as png image
			response.setContentType("image/png");
			ChartUtilities.writeChartAsPNG(out, chart, 300, 200);
		} catch (Exception e) {
			e.printStackTrace(System.out);
		} finally {
			out.close();
		}
	}
}
