/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JMXUtils.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.util;

import java.io.IOException;

import javax.management.remote.JMXConnector;

import org.objectweb.petals.tools.webadmin.Object.ServerConfiguration;

public class JMXUtils {

	/**
	 * Host for the JMX connection
	 */
	private String host = "";

	/**
	 * Port for the JMX connection
	 */
	private String jmxPort = "";

	/**
	 * Username for the JMX connection
	 */
	private String jmxLogin = "";

	/**
	 * Password the JMX connection
	 */
	private String jmxPassword = "";

	/**
	 * create a JMXUtils to communicate with a server of given parameters
	 * 
	 * @param host
	 * @param jmxPort
	 * @param jmxLogin
	 * @param jmxPassword
	 */
	public JMXUtils(String host, String jmxPort, String jmxLogin,
			String jmxPassword) {
		this.host = host;
		this.jmxPort = jmxPort;
		this.jmxLogin = jmxLogin;
		this.jmxPassword = jmxPassword;
	}

	/**
	 * create a JMXUtils to communicate with a server of given
	 * ServerConfiguration
	 * 
	 * @param serverConf
	 */
	public JMXUtils(ServerConfiguration serverConf) {
		this.host = serverConf.getHost();
		this.jmxPort = serverConf.getJmxPort();
		this.jmxLogin = serverConf.getJmxLogin();
		this.jmxPassword = serverConf.getJmxPassword();
	}

	/**
	 * Retrieve a JMX connector for a server with connection parameters given
	 * 
	 * @return a JMXConnector
	 * @throws IOException
	 */
	public static JMXConnector retrieveConnector(String host, String jmxPort,
			String jmxLogin, String jmxPassword) throws IOException {
		return JBIJMXConnectorUtil.getConnection(host, jmxPort, jmxLogin,
				jmxPassword);
	}

	/**
	 * Retrieve a JMX connector for a given server
	 * 
	 * @return a JMXConnector
	 * @throws IOException
	 */
	public static JMXConnector retrieveConnector(ServerConfiguration serverConf)
			throws IOException {
		return JBIJMXConnectorUtil.getConnection(serverConf.getHost(),
				serverConf.getJmxPort(), serverConf.getJmxLogin(), serverConf
						.getJmxPassword());
	}

	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public String getJmxPort() {
		return jmxPort;
	}

	public void setJmxPort(String jmxPort) {
		this.jmxPort = jmxPort;
	}

	public String getJmxLogin() {
		return jmxLogin;
	}

	public void setJmxLogin(String jmxLogin) {
		this.jmxLogin = jmxLogin;
	}

	public String getJmxPassword() {
		return jmxPassword;
	}

	public void setJmxPassword(String jmxPassword) {
		this.jmxPassword = jmxPassword;
	}

}
